SystemOrganization addCategory: #'Seaside-RenderLoop'!

WASessionContinuation subclass: #WARenderLoopContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

WARenderLoopContinuation subclass: #WAActionPhaseContinuation
	instanceVariableNames: 'renderContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAActionPhaseContinuation class methodsFor: 'private' stamp: 'jf 8/23/2011 16:20'!
defaultRenderContext
	^ nil! !

!WAActionPhaseContinuation class methodsFor: 'testing' stamp: 'jf 8/24/2011 12:07'!
isActionContinuation
	^ true! !

!WAActionPhaseContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:20'!
snapshot: aSnapshot
	^ self snapshot: aSnapshot renderContext: self defaultRenderContext! !

!WAActionPhaseContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:12'!
snapshot: aSnapshot renderContext: aRenderContext
	^ self basicNew
		initializeWithSnapshot: aSnapshot renderContext: aRenderContext;
		yourself! !

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 11:28'!
basicPerformAction
	"Do nothing by default. Subclasses can override."! !

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 13:52'!
continue
	| renderContinuation |
	renderContinuation := self createRenderContinuation.
	renderContinuation captureState.
	(self shouldRedirect) 
		ifTrue: [ self redirectToContinuation: renderContinuation ]
		ifFalse: [ renderContinuation handle: self requestContext ]! !

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 14:01'!
handleFiltered: aRequestContext
	self performAction.
	self continue! !

!WAActionPhaseContinuation methodsFor: 'initialization' stamp: 'jf 8/23/2011 16:10'!
initializeWithSnapshot: aSnapshot renderContext: aRenderContext
	self initializeWithSnapshot: aSnapshot.
	renderContext := aRenderContext! !

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 11:31'!
performAction
	self withNotificationHandlerDo: [ self basicPerformAction ].
	self startTasks! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 1/23/2009 12:05'!
renderContext
	^ renderContext! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 1/23/2009 12:06'!
renderContext: aRenderContext
	renderContext := aRenderContext! !

!WAActionPhaseContinuation methodsFor: 'private' stamp: 'jf 10/19/2008 12:41'!
shouldRedirect
	| request |
	request := self request.
	^ (request isGet and: [ request fields includesKey: '_n' ]) not! !

!WAActionPhaseContinuation methodsFor: 'private' stamp: 'jf 8/23/2011 11:32'!
startTasks
	"Make sure that any new Tasks get executed"
	(WAVisiblePresenterGuide client: WATaskVisitor new) start: self presenter! !

WAActionPhaseContinuation subclass: #WACallbackProcessingActionContinuation
	instanceVariableNames: 'jumpTo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WACallbackProcessingActionContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:23'!
snapshot: aSnapshot renderContext: aRenderContext
	aRenderContext isNil ifTrue: [ self error: 'Objects of this class require a RenderContext for callback processing.' ].
	^ super snapshot: aSnapshot renderContext: aRenderContext! !

!WACallbackProcessingActionContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 11:33'!
basicPerformAction
	super basicPerformAction.
	self renderContext callbacks handle: self requestContext.! !

!WACallbackProcessingActionContinuation methodsFor: 'public' stamp: 'pmm 8/27/2011 17:23'!
jumpToAnchor: aString
	jumpTo := aString! !

!WACallbackProcessingActionContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 11:33'!
performAction
	"Ajax requests modify the captured state - though this behaviour has been heavily debated"
	[ super performAction ] ensure: [
		self request isXmlHttpRequest ifTrue: [ self captureState ] ]! !

!WACallbackProcessingActionContinuation methodsFor: 'private' stamp: 'pmm 8/27/2011 17:29'!
redirectToContinuation: aSessionContinuation
	self respond: [ :response |
		| url |
		url := aSessionContinuation registerForUrl.
		jumpTo isNil ifFalse: [
			url fragment: jumpTo ].
		response redirectTo: url ]! !

WAActionPhaseContinuation subclass: #WAInitialRenderLoopContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAInitialRenderLoopContinuation commentStamp: 'pmm 8/25/2011 16:59' prior: 0!
I'm the initial continuation of a render loop. I just render the initial page. Subclasses may want to override #shouldRedirect and answer true so that a redirect happens before dispalying the first page.!

!WAInitialRenderLoopContinuation methodsFor: 'processing' stamp: 'jf 8/24/2011 11:09'!
basicPerformAction
	| root |
	root := self createRoot.
	self session properties at: #presenter put: root.
	self prepareRoot: root! !

!WAInitialRenderLoopContinuation methodsFor: 'creational' stamp: 'jf 8/24/2011 11:06'!
createRoot
	^ self rootDecorationClasses
		inject: self rootClass new
		into: [ :component :decorationClass |
			component 
				addDecoration: decorationClass new;
				yourself ]! !

!WAInitialRenderLoopContinuation methodsFor: 'processing' stamp: 'jf 8/24/2011 11:06'!
prepareRoot: aComponent
	(WAAllPresenterGuide client:
		(WAInitialRequestVisitor request: self requestContext request))
		start: aComponent.
	(WAVisiblePresenterGuide client: (WATaskVisitor new)) start: aComponent! !

!WAInitialRenderLoopContinuation methodsFor: 'accessing' stamp: 'jf 8/24/2011 11:06'!
rootClass
	^ self application preferenceAt: #rootClass! !

!WAInitialRenderLoopContinuation methodsFor: 'accessing' stamp: 'jf 8/24/2011 11:06'!
rootDecorationClasses
	^ self application preferenceAt: #rootDecorationClasses! !

!WAInitialRenderLoopContinuation methodsFor: 'private' stamp: 'jf 8/24/2011 14:11'!
shouldRedirect
	"This should probably be made configurable, but for backwards compatibility with Seaside 3.0,
	where we do not redirect when you enter a session, we return false."
	
	^ false! !

WAActionPhaseContinuation subclass: #WAPluggableActionContinuation
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAPluggableActionContinuation commentStamp: 'jf 8/23/2011 12:09' prior: 0!
This continuation executes an action (any class that implements #value or #value:). If possible, the renderContext is passed in as an argument. When the action is complete, if a response hasn't been returned, control is passed to a render continuation.'

Instance Variables:
	action	<BlockClosure | BlockContext | GRDelayedSend | MessageSend | WAContinuation | WAPartialContinuation>!

!WAPluggableActionContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:19'!
action: aValuable
	^ self snapshot: self defaultSnapshot renderContext: self defaultRenderContext action: aValuable! !

!WAPluggableActionContinuation class methodsFor: 'private' stamp: 'jf 8/23/2011 16:17'!
defaultAction
	^ GRDelayedSend empty! !

!WAPluggableActionContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:16'!
snapshot: aSnapshot renderContext: aRenderContext
	^ self snapshot: aSnapshot renderContext: aRenderContext action: self defaultAction! !

!WAPluggableActionContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:16'!
snapshot: aSnapshot renderContext: aRenderContext action: aValuable
	^ self basicNew
		initializeWithSnapshot: aSnapshot renderContext: aRenderContext action: aValuable;
		yourself! !

!WAPluggableActionContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 12:02'!
basicPerformAction
	^ action valueWithPossibleArguments: (Array with: renderContext)! !

!WAPluggableActionContinuation methodsFor: 'initialization' stamp: 'jf 8/23/2011 16:15'!
initializeWithSnapshot: aSnapshot renderContext: aRenderContext action: aValuable
	self initializeWithSnapshot: aSnapshot renderContext: aRenderContext.
	action := aValuable! !

!WARenderLoopContinuation class methodsFor: 'testing' stamp: 'jf 8/24/2011 12:06'!
isActionContinuation
	^ false! !

!WARenderLoopContinuation class methodsFor: 'testing' stamp: 'jf 8/24/2011 12:06'!
isRenderContinuation
	^ false! !

!WARenderLoopContinuation class methodsFor: 'testing' stamp: 'jf 8/24/2011 12:06'!
isRenderLoopContinuation
	^ true! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 8/23/2011 16:08'!
createActionContinuation
	"Answer a new action phase continuation."

	^ (self application preferenceAt: #actionPhaseContinuationClass) snapshot: self states! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 8/23/2011 16:27'!
createActionContinuationWithContext: aRenderContext
	"Answer a new action phase continuation."

	^ (self application preferenceAt: #actionPhaseContinuationClass) snapshot: self states renderContext: aRenderContext! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 3/4/2009 09:49'!
createRenderContinuation
	"Answer a new render phase continuation."

	^ (self application preferenceAt: #renderPhaseContinuationClass) new! !

!WARenderLoopContinuation methodsFor: 'testing' stamp: 'jf 8/24/2011 12:07'!
isActionContinuation
	^ self class isActionContinuation! !

!WARenderLoopContinuation methodsFor: 'testing' stamp: 'jf 8/24/2011 12:07'!
isRenderContinuation
	^ self class isRenderContinuation! !

!WARenderLoopContinuation methodsFor: 'accessing' stamp: 'jf 2/20/2009 11:58'!
presenter
	"Answer the root presenter instance for the current session."

	^ self session presenter! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'pmm 8/30/2009 14:50'!
toPresenterSendRoot: anHtmlRoot
	(WAVisiblePresenterGuide client:
		(WAUpdateRootVisitor root: anHtmlRoot))
		start: self presenter! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'lr 2/21/2010 20:52'!
updateRoot: anHtmlRoot
	anHtmlRoot meta contentType: self application contentType.
	anHtmlRoot meta contentScriptType: WAMimeType textJavascript.
	anHtmlRoot htmlAttributes
		at: 'xmlns' put: 'http://www.w3.org/1999/xhtml';
		at: 'xml:lang' put: 'en';
		at: 'lang' put: 'en'.
	anHtmlRoot beXhtml10Strict.
	anHtmlRoot title: 'Seaside'.
	self application libraries 
		do: [ :each | each default updateRoot: anHtmlRoot ].
	self session updateRoot: anHtmlRoot! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 2/20/2009 11:59'!
updateStates: aSnapshot
	super updateStates: aSnapshot.
	(WAAllPresenterGuide client:
		(WAUpdateStatesVisitor snapshot: aSnapshot))
		start: self presenter! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'lr 9/8/2009 12:50'!
updateUrl: aUrl
	super updateUrl: aUrl.
	(WAVisiblePresenterGuide client: (WAUpdateUrlVisitor url: aUrl))
		start: self presenter! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'pmm 9/1/2011 09:31'!
withNotificationHandlerDo: aBlock
	| currentHandler |
	currentHandler := WACurrentExceptionHandler value.
	^ currentHandler isNil
		ifTrue: [
			aBlock on: WARenderNotification do: [ :n | ] ]
		ifFalse: [
			"For platforms with ANSI exceptions we have to nest exception handlers
			
			with ANSI exceptions
			When signaling an exception from an exception handler block the look up of
			the new exception handler starts in the exception handler block.
			
			without ANSI exceptions
			When signaling an exception from an exception handler block the look up of
			the new exception handler starts where the original exception was signaled.
			
			platforms without ANSI exceptions: Pharo, Squeak
			platforms with ANSI exception: VAST, VW"
			[ currentHandler handleExceptionsDuring: aBlock ]
				on: WARenderNotification
				 do: [ :n | ] ]
		! !

WARenderLoopContinuation subclass: #WARenderPhaseContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderPhaseContinuation class methodsFor: 'testing' stamp: 'jf 8/24/2011 12:07'!
isRenderContinuation
	^ true! !

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'lr 7/25/2011 20:08'!
createHtmlRootWithContext: aRenderContext
	^ WAHtmlRoot context: aRenderContext! !

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 1/23/2009 11:54'!
createRenderContext
	| renderContext |
	renderContext := WARenderContext new.
	
	"Wrap the default visitor with a rendering guide"
	renderContext visitor: (WARenderingGuide client: renderContext visitor).
	
	^ renderContext
		! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'pmm 8/27/2011 17:29'!
handleFiltered: aRequestContext
	self respond: [ :response | self processRendering: response ].

	"The render phase should have returned a response"
	WAMissingResponseError signal! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 8/23/2011 16:27'!
processRendering: aResponse 
	| document actionContinuation renderContext actionUrl actionBaseUrl htmlRoot |
	"Prepare the response"
	aResponse contentType: self application contentType.

	"Create objects we need"
	document := self requestContext newDocument.
	renderContext := self createRenderContext.
	actionContinuation := self createActionContinuationWithContext: renderContext.
	
	"Create URL"
	actionUrl := actionContinuation registerForUrl: false.
	actionBaseUrl := actionUrl copy.
	self updateUrl: actionUrl.

	"Hook objects together"
	renderContext
		document: document;
		actionUrl: actionUrl;
		actionBaseUrl: actionBaseUrl;
		resourceUrl: self application resourceBaseUrl.
	actionContinuation renderContext: renderContext.
	htmlRoot := self createHtmlRootWithContext: renderContext.
	self updateRoot: htmlRoot.
	self toPresenterSendRoot: htmlRoot.

	"Render"
	document open: htmlRoot.
	self presenter renderWithContext: renderContext.
	document close.
	renderContext destroy! !

!WASessionContinuation class methodsFor: '*seaside-renderloop' stamp: 'jf 8/24/2011 12:08'!
isRenderLoopContinuation
	^ false! !

!WASessionContinuation methodsFor: '*seaside-renderloop' stamp: 'jf 8/24/2011 12:08'!
isRenderLoopContinuation
	^ self class isRenderLoopContinuation! !

!WASession methodsFor: '*seaside-renderloop' stamp: 'jf 2/20/2009 10:12'!
presenter
	^ self properties at: #presenter ifAbsent: [ nil ]! !

WAError subclass: #WAMissingResponseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAMissingResponseError methodsFor: 'accessing' stamp: 'jf 2/20/2009 18:09'!
seasidePossibleCauses
	^ #(
		'A Component attempted to use #call: or a similar method while rendering. Calling and answering can only be done in a callback.'
	)! !

!GRPackage class methodsFor: '*seaside-renderloop' stamp: 'lr 2/17/2010 15:13'!
seasideRenderLoop
	^ self new
		name: 'Seaside-RenderLoop';
		description: 'Defines the low-level rendering and callback processing functionality.';
		addDependency: 'Seaside-Session';
		addDependency: 'Seaside-Component';
		url: #seasideUrl;
		yourself! !

WASystemConfiguration subclass: #WARenderLoopConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopConfiguration commentStamp: 'jf 7/22/2008 12:32' prior: 0!
WARenderLoopConfiguration defines attributes required by the Render Loop classes (see the Seaside-Core-RenderLoop category). Any application using these classes (e.g. WARenderLoopMain) should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 8/24/2011 12:01'!
actionPhaseContinuationClasses
	^ self continuationClasses select: [ :each | each isRenderLoopContinuation and: [ each isActionContinuation ] ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 8/24/2011 12:00'!
continuationClasses
	^ WAApplicationConfiguration instance continuationClasses! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'pmm 8/25/2011 14:47'!
describeOn: config
	(config list: #actionPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s action phase.';
		options: [ self actionPhaseContinuationClasses ];
		default: WACallbackProcessingActionContinuation.
	(config list: #renderPhaseContinuationClass)
		beAdvanced;
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s render phase.';
		options: [ self renderPhaseContinuationClasses ];
		default: WARenderPhaseContinuation.
	(config list: #rootClass)
		comment: 'The root component of this seaside application.';
		options: [ self rootClasses ].
	(config collection: #rootDecorationClasses)
		comment: 'The decoration clases that should wrap the root component.';
		options: [ self rootDecorationClasses ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 8/24/2011 12:01'!
renderPhaseContinuationClasses
	^ self continuationClasses select: [ :each | each isRenderLoopContinuation and: [ each isRenderContinuation ] ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 7/29/2008 14:30'!
rootClasses
	^ (WAComponent allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 9/14/2008 12:58'!
rootDecorationClasses
	^ (WADecoration allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

WADynamicVariable subclass: #WACurrentCallbackProcessingActionContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!
